@@ -24,12 +24,14 @@ from swift.common.utils import (
     clean_content_type, config_true_value, Timestamp, public,
     close_if_possible, closing_if_possible)
 from swift.common.constraints import check_metadata, check_object_creation
+from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.middleware.versioned_writes import DELETE_MARKER_CONTENT_TYPE
 from swift.common.swob import HTTPAccepted, HTTPBadRequest, HTTPNotFound, \
     HTTPConflict, HTTPPreconditionFailed, HTTPRequestTimeout, \
     HTTPUnprocessableEntity, HTTPClientDisconnect, HTTPCreated, \
     HTTPNoContent, Response, HTTPInternalServerError, multi_range_iterator
-from swift.common.request_helpers import is_sys_or_user_meta
+from swift.common.request_helpers import is_sys_or_user_meta, \
+    is_object_transient_sysmeta
 from swift.proxy.controllers.base import set_object_info_cache, \
         delay_denial, cors_validation
 from swift.proxy.controllers.obj import check_content_type
@@ -215,6 +217,7 @@ class ObjectController(BaseObjectController):
         if properties:
             for k, v in properties.iteritems():
                 if is_sys_or_user_meta('object', k) or \
+                        is_object_transient_sysmeta(k) or \
                         k.lower() in self.allowed_headers:
                     resp.headers[str(k)] = v
         resp.headers['etag'] = metadata['hash'].lower()
@@ -239,7 +242,8 @@ class ObjectController(BaseObjectController):
         metadata = {}
         metadata.update(
             (k.lower(), v) for k, v in headers.iteritems()
-            if is_sys_or_user_meta('object', k))
+            if is_sys_or_user_meta('object', k) or
+            is_object_transient_sysmeta(k))
         for header_key in self.allowed_headers:
             if header_key in headers:
                 headers_lower = header_key.lower()
@@ -312,6 +316,8 @@ class ObjectController(BaseObjectController):
 
         self._update_content_type(req)
 
+        self._update_x_timestamp(req)
+
         # check constraints on object name and request headers
         error_response = check_object_creation(req, self.object_name) or \
             check_content_type(req)
@@ -321,8 +327,6 @@ class ObjectController(BaseObjectController):
         if req.headers.get('Oio-Copy-From'):
             return self._link_object(req)
 
-        self._update_x_timestamp(req)
-
         data_source = req.environ['wsgi.input']
         if req.content_length:
             data_source = ExpectedSizeReader(data_source, req.content_length)
@@ -368,6 +372,8 @@ class ObjectController(BaseObjectController):
         else:
             ranges = None
 
+        headers = self._prepare_headers(req)
+        metadata = self.load_object_metadata(headers)
         oio_headers = {'X-oio-req-id': self.trans_id}
         # FIXME(FVE): use object_show, cache in req.environ
         props = storage.object_get_properties(from_account, container, obj)
@@ -410,7 +416,8 @@ class ObjectController(BaseObjectController):
             storage.object_fastcopy(
                 from_account, container, obj,
                 self.account_name, self.container_name, self.object_name,
-                headers=oio_headers)
+                headers=oio_headers, properties=metadata,
+                properties_directive='REPLACE')
         # TODO(FVE): this exception catching block has to be refactored
         # TODO check which ones are ok or make non sense
         except exceptions.Conflict:
@@ -446,6 +453,17 @@ class ObjectController(BaseObjectController):
         resp = HTTPCreated(request=req, etag=checksum)
         return resp
 
+    def _get_footers(self, req):
+        """
+        Get extra metadata that may be generated during upload by some
+        middlewares (e.g. checksum of cyphered data).
+        """
+        footers = HeaderKeyDict()
+        footer_callback = req.environ.get(
+            'swift.callback.update_footers', lambda _footer: None)
+        footer_callback(footers)
+        return footers
+
     def _store_object(self, req, data_source, headers):
         content_type = req.headers.get('content-type', 'octet/stream')
         storage = self.app.storage
@@ -474,11 +492,20 @@ class ObjectController(BaseObjectController):
         metadata = self.load_object_metadata(headers)
         oio_headers = {'X-oio-req-id': self.trans_id}
         try:
+            # FIXME(FVE): use 'properties' instead of 'metadata'
+            # as soon as we require oio>=4.2.0
             _chunks, _size, checksum = storage.object_create(
                 self.account_name, self.container_name,
                 obj_name=self.object_name, file_or_path=data_source,
                 mime_type=content_type, policy=policy, headers=oio_headers,
                 etag=req.headers.get('etag', '').strip('"'), metadata=metadata)
+            # TODO(FVE): when oio-sds supports it, do that in a callback
+            # passed to object_create (or whatever upload method supports it)
+            footer_md = self.load_object_metadata(self._get_footers(req))
+            if footer_md:
+                storage.object_set_properties(
+                    self.account_name, self.container_name, self.object_name,
+                    properties=footer_md)
         except exceptions.Conflict:
             raise HTTPConflict(request=req)
         except exceptions.PreconditionFailed:
@@ -536,20 +563,6 @@ class ObjectController(BaseObjectController):
             else:
                 req.content_type_manually_set = False
 
-    def _update_x_timestamp(self, req):
-        if 'x-timestamp' in req.headers:
-            try:
-                req_timestamp = Timestamp(req.headers['X-Timestamp'])
-            except ValueError:
-                raise HTTPBadRequest(
-                    request=req, content_type='text/plain',
-                    body='X-Timestamp should be a UNIX timestamp float value; '
-                         'was %r' % req.headers['x-timestamp'])
-            req.headers['X-Timestamp'] = req_timestamp.internal
-        else:
-            req.headers['X-Timestamp'] = Timestamp(time.time()).internal
-        return None
-
     @public
     @cors_validation
     @delay_denial
